﻿using System;
using System.ComponentModel;
using System.Text;
using System.Windows;
using System.Windows.Threading;
using Microsoft.Expression.Framework;
using Oxage.Common;
using Microsoft.Windows.Design.Interaction;

namespace Oxage.Rasterizer
{
	public class RasterContractService : IContractService, INotifyPropertyChanged
	{
		#region Constructors
		public RasterContractService()
		{
			this.ProjectType = ProjectTypes.Other;
		}

		public RasterContractService(IApplicationService appService) : this()
		{
			RegisterApplicationService(appService);
		}
		#endregion

		#region Public properties
		private IApplicationService appService;
		public IApplicationService AppService
		{
			get
			{
				return this.appService;
			}
		}
		#endregion

		#region Public methods
		public void RegisterApplicationService(IApplicationService appService)
		{
			this.appService = appService;
			InitializeAddin();
		}
		#endregion

		#region Protected methods
		protected void InitializeAddin()
		{
			if (appService == null)
			{
				ReportError(new Exception("appService is null, plugin cannot be loaded!"));
			}
			else
			{
				//TODO: Find a better trigger instead of timer
				DispatcherTimer timer = new DispatcherTimer();
				timer.Interval = TimeSpan.FromSeconds(1);
				timer.Tick += (sender, e) =>
				{
					try
					{
						LoadDocumentElement();
					}
					catch (Exception ex)
					{
						OnDebugMessage(ex.Message);
					}
				};
				timer.Start();
			}
		}

		protected void LoadDocumentElement()
		{
			if (this.DocumentPath != appService.DocumentService.ActiveDocument.DocumentReference.Path)
			{
				#region Find project type
				if (appService.ViewService.ActiveView is Microsoft.Expression.DesignSurface.View.WPFSceneView)
				{
					this.ProjectType = ProjectTypes.WPF;
				}
				else if (appService.ViewService.ActiveView is Microsoft.Expression.DesignSurface.View.SilverlightSceneView)
				{
					this.ProjectType = ProjectTypes.Silverlight;
				}
				else
				{
					this.ProjectType = ProjectTypes.Other;
				}
				#endregion

				#region Find visual content
				if (this.ProjectType == ProjectTypes.WPF)
				{
					var scene = appService.ViewService.ActiveView as Microsoft.Expression.DesignSurface.View.WPFSceneView;
					if (scene != null)
					{
						var artboard = scene.Artboard.ContentArea as Microsoft.Expression.DesignSurface.UserInterface.ArtboardBorder;
						if (artboard != null)
						{
							this.DocumentContent = artboard.Child as FrameworkElement;
							this.DocumentPath = appService.DocumentService.ActiveDocument.DocumentReference.Path;
							OnDocumentChanged();
						}
					}
				}
				else
				{
					//Experimental, does not work properly with Silverlight
					var scene = appService.ViewService.ActiveView as Microsoft.Expression.DesignSurface.View.SceneView;
					if (scene != null)
					{
						this.DocumentContent = scene.Artboard.DesignerView.Child as FrameworkElement;
						this.DocumentPath = appService.DocumentService.ActiveDocument.DocumentReference.Path;
						OnDocumentChanged();
					}
				}
				#endregion
			}

			if (Settings.Debug)
			{
				#region Build debug message
				StringBuilder builder = new StringBuilder();

				builder.AppendFormat("DocumentPath: {0}", this.DocumentPath).AppendLine();

				if (appService == null)
				{
					builder.AppendLine("ApplicationService is null!");
				}
				else if (appService.DocumentService == null)
				{
					builder.AppendLine("DocumentService is null!");
				}
				else if (appService.DocumentService.ActiveDocument == null)
				{
					builder.AppendLine("DocumentService.ActiveDocument is null!");
				}
				else if (appService.DocumentService.ActiveDocument.DocumentReference == null)
				{
					builder.AppendLine("DocumentService.ActiveDocument.DocumentReference is null!");
				}
				else
				{
					builder.AppendFormat("DocumentReference.Path: {0}", appService.DocumentService.ActiveDocument.DocumentReference.Path).AppendLine();
				}

				if (appService != null)
				{
					if (appService.ViewService == null)
					{
						builder.AppendLine("ViewService is null!");
					}
					else if (appService.ViewService.ActiveView == null)
					{
						builder.AppendLine("ViewService.ActiveView is null!");
					}
					else
					{
						builder.AppendFormat("ActiveView: {0}", appService.ViewService.ActiveView.GetType().ToString()).AppendLine();

						var scene = appService.ViewService.ActiveView as Microsoft.Expression.DesignSurface.View.SceneView;
						if (scene != null)
						{
							if (scene.Artboard == null)
							{
								builder.AppendLine("Artboard is null!");
							}
							else
							{
								builder.AppendFormat("Artboard: {0}", scene.Artboard.GetType().ToString()).AppendLine();

								var artboard = scene.Artboard.ContentArea as Microsoft.Expression.DesignSurface.UserInterface.ArtboardBorder;
								if (artboard != null)
								{
									builder.AppendFormat("Artboard content: {0}", artboard.Child != null ? artboard.Child.GetType().ToString() : "null").AppendLine();
								}
							}
						}
					}
				}

				OnDebugMessage(builder.ToString());
				#endregion
			}
		}

		protected void ReportError(Exception error)
		{
			if (appService != null && error != null)
			{
				//Display message in application's logging window
				appService.MessageLoggingService.WriteLine(error.Message);
			}

			if (Settings.Debug && error != null)
			{
				MessageBox.Show(error.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
			}
		}
		#endregion

		#region IContractService Members
		public string DocumentPath
		{
			get
			{
				return this.documentPath;
			}
			set
			{
				if (this.documentPath != value)
				{
					this.documentPath = value;
					OnPropertyChanged("DocumentPath");
				}
			}
		}
		private string documentPath;

		public FrameworkElement DocumentContent
		{
			get
			{
				return this.documentContent;
			}
			set
			{
				if (this.documentContent != value)
				{
					this.documentContent = value;
					OnPropertyChanged("DocumentContent");
				}
			}
		}
		private FrameworkElement documentContent;

		public ProjectTypes ProjectType
		{
			get
			{
				return this.projectType;
			}
			protected set
			{
				if (this.projectType != value)
				{
					this.projectType = value;
					OnPropertyChanged("ProjectType");
				}
			}
		}
		private ProjectTypes projectType;

		public void WriteLine(string message)
		{
			if (this.appService != null)
			{
				this.appService.MessageLoggingService.WriteLine(message);
			}
		}

		public event EventHandler DocumentChanged;
		protected virtual void OnDocumentChanged()
		{
			if (DocumentChanged != null)
			{
				DocumentChanged(this, new EventArgs());
			}
		}

		public event EventHandler<EventArgs<string>> DebugMessage;
		protected virtual void OnDebugMessage(string message)
		{
			if (DebugMessage != null)
			{
				DebugMessage(this, new EventArgs<string>(message));
			}
		}
		#endregion

		#region INotifyPropertyChanged Members
		public event PropertyChangedEventHandler PropertyChanged;
		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
		#endregion
	}
}
